@@ -1,6 +1,8 @@
 from typing import Dict, List, Union, Tuple
 import json
 import os
+from uuid import uuid4
+
 from .lectric_types import HashAlgo
 from requests import Response
 from functools import wraps
@@ -12,6 +14,8 @@ from .models.collection_in_spec import CollectionInSpec
 from .models.query_spec import QuerySpec
 from .models.vector_query_spec import VectorQuerySpec
 from .models.query_response import QueryResponse
+from .models.collection_metadata import CollectionMetadata
+from .models.schema_info_exact_schema_info_get_response_schema_info_exact_schema_info_get import SchemaInfoExactSchemaInfoGetResponseSchemaInfoExactSchemaInfoGet
 
 from .helpers import *
 from .client import AuthenticatedClient
@@ -31,11 +35,13 @@ from .api.collection import (
     drop_collection_collection_delete,
     sizeof_collection_collection_size_name_get,
     delete_entities_collection_entities_delete,
-    sample_collection_sample_get
+    sample_collection_sample_get,
+    get_url_collection_get_url_get
 )
 
 # ingest
 from .api.ingest import ingest_ingest_post
+from .api.ingest import ingest_w_file_ingest_file_post
 
 # index
 from .api.index import create_index_create_post, delete_index_collection_name_field_name_delete
@@ -62,12 +68,17 @@ from .api.exact import (
     raw_query_exact_experimental_raw_query_post,
     ping_exact_ping_post,
     schema_info_exact_schema_info_get,
-    get_url_exact_get_url_get
+    delete_entries_exact_entries_delete,
+    delete_entries_by_file_exact_entries_by_file_delete,
+    select_entries_exact_select_post,
+    get_collection_metadata_exact_collection_metadata_get
 )
 
 from .models.body_put_entry_by_file_exact_hash_collection_name_file_put import BodyPutEntryByFileExactHashCollectionNameFilePut
 from .models.body_lookup_by_file_exact_lookup_file_collection_name_post import BodyLookupByFileExactLookupFileCollectionNamePost
-
+from .models.body_ingest_w_file_ingest_file_post import BodyIngestWFileIngestFilePost
+from .models.body_delete_entries_exact_entries_delete import BodyDeleteEntriesExactEntriesDelete
+from .models.body_delete_entries_by_file_exact_entries_by_file_delete import BodyDeleteEntriesByFileExactEntriesByFileDelete
 
 def propagate_exception(func):
     @wraps(func)
@@ -75,10 +86,13 @@ def propagate_exception(func):
 
         response = func(*args, **kwargs)
 
-        if response.status_code == HTTPStatus.OK:
-            return response.parsed
+        if response:
+            if response.status_code == HTTPStatus.OK:
+                return response.parsed
+            else:
+                raise RuntimeError(f"HTTP Code: {response.status_code}, {response}")
         else:
-            raise RuntimeError(f"HTTP Code: {response.status_code}, {response}")
+            raise RuntimeError("No response")
 
     return propagator
 
@@ -137,14 +151,18 @@ class LectricClient:
     ########################## collections ###############################
 
     @propagate_exception
-    def create_collection(self, coll_in_spec: CollectionInSpec, with_reverse_lookup: bool=False,
-                                                        with_dupe_counter: bool=False) -> Collection:
+    def create_collection(self, coll_in_spec: CollectionInSpec,
+                                    with_reverse_lookup: bool=False,
+                                    with_dupe_counter: bool=False,
+                                    hash_algo: HashAlgo =HashAlgo.md5
+                                    ) -> Collection:
         """Create an exact OR vector collection. If `coll_in_spec.approx == True` then the collection created will be a vector/approximate collection; if not it will be an exact one
 
         Args:
             coll_in_spec (CollectionInSpec): The collection specification to be created
             with_reverse_lookup (bool): Only relevant to exact collections. Creates a sidecar collection to do reverse lookups. Defaults to False to False.
             with_dupe_counter (bool): Only relevant to exact collections. Adds a counting attribute to each member of the collection for tracking and counting.
+            hash_algo (HashAlgo): The hash algorithm to use for exact collections. Default is HashAlgo.md5
 
         Returns:
             Collection: An object representing the created collection
@@ -154,7 +172,8 @@ class LectricClient:
         return create_collection_exact_create_collection_post.sync_detailed(client=self.client,
                                                                                 json_body=coll_in_spec,
                                                                                 with_reverse_lookup=with_reverse_lookup,
-                                                                                with_dupe_counter=with_dupe_counter)
+                                                                                with_dupe_counter=with_dupe_counter,
+                                                                                hash_algo=hash_algo.name)
 
 
     @propagate_exception
@@ -241,18 +260,45 @@ class LectricClient:
 
     @propagate_exception
     def delete_entities(self, collection_name: str,
-                ids: Union[List[str], List[int]]) -> None:
-        """Delete entities by Primary key ID
+                ids: Union[List[str], List[int]]=None,
+                urls: List[str]=None,
+                file: Union[BinaryIO, bytes, str]=None,
+                is_approx: bool=True) -> None:
+        """Delete entities by Primary key ID, or (for exact:) by file or URI
 
         Args:
             collection_name (str): The collection name
-            ids (Union[List[str], List[int]]): The list of IDs for which is to happen
+            ids (Union[List[str], List[int]], optional): The list of IDs for which is to happen
+            urls (List[str], optional): A list of URLs to be hashed & potentially deleted. Defaults to None.
+            file (Union[BinaryIO, bytes, str], optional): File to be hashed & potentially deleted. Defaults to None.
+            is_approx (bool): Is the collection approximate or exact?
 
         Returns:
             _type_: None
         """
-        return delete_entities_collection_entities_delete.sync_detailed(
+        if is_approx:
+            return delete_entities_collection_entities_delete.sync_detailed(
                 client=self.client, name=collection_name, json_body=ids)
+        else:
+            # Exact
+            if ids:
+                return delete_entries_exact_entries_delete.sync_detailed(
+                    client=self.client, collection_name=collection_name,
+                    json_body=BodyDeleteEntriesExactEntriesDelete(entry_ids=ids)
+                )
+            if urls:
+                return delete_entries_exact_entries_delete.sync_detailed(
+                    client=self.client, collection_name=collection_name,
+                    json_body=BodyDeleteEntriesExactEntriesDelete(urls=urls),
+                )
+            if file:
+                return delete_entries_by_file_exact_entries_by_file_delete.sync_detailed(
+                    client=self.client, collection_name=collection_name,
+                    multipart_data=BodyDeleteEntriesByFileExactEntriesByFileDelete(
+                        file=fastAPIFile(payload=get_tmp_file(file), file_name=str(uuid4()),
+                                                mime_type="image/jpg")
+                    )
+                )
 
 
     @propagate_exception
@@ -300,6 +346,33 @@ class LectricClient:
         """
         return ingest_ingest_post.sync_detailed(client=self.client, json_body=data)
 
+
+    @propagate_exception
+    def ingest_with_file(self, collection_name: str, json_input_data: str,
+                    file: Union[str, bytes, BinaryIO]=None) -> None:
+        """Ingest *a single* element with a file into a collection
+
+        Args:
+            collection_name (str): The unique name of the collection
+            json_input_data (str): A json string formatted input identical to that of InputData
+            file (Union[str, bytes, BinaryIO], optional): _description_. Defaults to None.
+
+        Returns:
+            _type_: None
+        """
+
+        file_name: str = "tmpfile"
+        if isinstance(file, str):
+            try:
+                file_name = os.path.basename(file)
+            except:
+                pass # we did our best \_(ツ)_/¯
+
+        return ingest_w_file_ingest_file_post.sync_detailed(client=self.client, multipart_data=BodyIngestWFileIngestFilePost(
+            file=fastAPIFile(payload=get_tmp_file(file), file_name=file_name, mime_type="image/jpg")),
+            collection_name=collection_name, json_input_data=json_input_data
+        )
+
     @propagate_exception
     def create_index(self, index_spec: IndexInSpec) -> None:
         """Create an index for one field within the collection
@@ -350,24 +423,24 @@ class LectricClient:
     @propagate_exception
     def put_exact(self, collection_name: str,
                     foreign_key: Union[str, int]=None,
-                    algo: str=HashAlgo.md5.name,
                     file: Union[str, bytes, BinaryIO]=None,
                     uri: str=None,
-                    metadata: str=None,
+                    metadata: dict=None,
                     upsert: bool = False,
-                    timestamp: str = None
+                    timestamp: str = None,
+                    store_raw_data: bool = False
                     ) -> str:
         """Put an entry into an exact store collection
 
         Args:
             collection_name (str): The collection name
-            foreign_key (Union[str, int], optional): Foreign key
-            algo (str, optional): The hashing algorithm to use. Defaults to "md5".
+            foreign_key (Union[str, int], optional): Foreign. key Defaults to None.
             file (Union[str, bytes, BinaryIO], optional): The file containing the data. Defaults to None.
             uri (str, optional): The URI of input data. Defaults to None.
-            metadata (str, optional): Additional metadata to store with entry
-            upsert (bool, optional): Update entry if it already exists
-            timestamp (str, optional): Timestamp to associate with entry
+            metadata (dict, optional): Dictionary of additional metadata to store with entry. Defaults to None.
+            upsert (bool, optional): Update entry if it already exists. Defaults to None.
+            timestamp (str, optional): Timestamp to associate with entry. Defaults to None.
+            store_raw_data(bool, optional): Store the raw data directly in lectric. Defaults to False.
 
         Raises:
             RuntimeError: When input file or URI is unprocessable
@@ -378,40 +451,29 @@ class LectricClient:
         if uri:
             return put_entry_by_uri_exact_hash_collection_name_uri_put.sync_detailed(
                 collection_name=collection_name, client=self.client,
-                foreign_key=foreign_key, uri=uri, algo=algo,
+                foreign_key=foreign_key, uri=uri,
                 upsert=upsert,
-                metadata=metadata,
-                timestamp=timestamp
-            )
+                metadata=None if not metadata else json.dumps(metadata),
+                timestamp=timestamp,
+                store_raw_data=store_raw_data)
         elif file:
-            if is_file_like(file):
-                tmp_file = file
-            elif isinstance(file, str):
-                if not os.path.exists(file):
-                    raise RuntimeError("`file` arg as str must be the path to file")
-                tmp_file = open(file, "rb")
-
-            elif isinstance(file, bytes): # Assume encoded JPG
-                tmp_file = to_tmpfile(file)
-            else:
-                raise RuntimeError("Unable to process `file` argument, unsupported input type")
             return put_entry_by_file_exact_hash_collection_name_file_put.sync_detailed(
                 collection_name=collection_name, client=self.client,
                 multipart_data=BodyPutEntryByFileExactHashCollectionNameFilePut(
-                        file=fastAPIFile(payload=tmp_file, file_name="tmpfile", mime_type="image/jpg")
-                    ), foreign_key = foreign_key, algo=algo, metadata=metadata, upsert=upsert, timestamp=timestamp
+                        file=fastAPIFile(payload=get_tmp_file(file), file_name="tmpfile", mime_type="image/jpg")
+                    ), foreign_key = foreign_key, metadata=None if not metadata else json.dumps(metadata),
+                        upsert=upsert, timestamp=timestamp,
+                        store_raw_data=store_raw_data
                 )
 
     @propagate_exception
     def lookup_exact(self, collection_name: str,
-                    algo: str = HashAlgo.md5.name,
                     file: Union[str, bytes, BinaryIO]=None,
                     uri: str=None) -> Union[List[str], None]:
         """Lookup exact entry in a collection
 
         Args:
             collection_name (str): The name of the collection
-            algo (str, optional): The hash algorithm to use (Should match insertion hashing algorithm). Defaults to "md5".
             file (Union[str, bytes, BinaryIO], optional): Input file data. Defaults to None.
             uri (str, optional): Input URI data. Defaults to None.
 
@@ -423,25 +485,13 @@ class LectricClient:
         """
         if uri:
             return lookup_by_uri_exact_lookup_hash_uri_collection_name_get.sync_detailed(
-                collection_name=collection_name, client=self.client, uri=uri, algo=algo
+                collection_name=collection_name, client=self.client, uri=uri,
             )
         elif file:
-            if is_file_like(file):
-                tmp_file = file
-            elif isinstance(file, str):
-                if not os.path.exists(file):
-                    raise RuntimeError("`file` arg as str must be the path to file")
-                tmp_file = open(file, "rb")
-
-            elif isinstance(file, bytes): # Assume encoded JPG
-                tmp_file = to_tmpfile(file)
-            else:
-                raise RuntimeError("Unable to process `file` argument, unsupported input type")
-
             return lookup_by_file_exact_lookup_file_collection_name_post.sync_detailed(
-                collection_name=collection_name, client=self.client, algo=algo,
+                collection_name=collection_name, client=self.client,
                 multipart_data=BodyLookupByFileExactLookupFileCollectionNamePost(
-                        file=fastAPIFile(payload=tmp_file, file_name="tmpfile", mime_type="image/jpg")
+                        file=fastAPIFile(payload=get_tmp_file(file), file_name="tmpfile", mime_type="image/jpg")
                     )
                 )
 
@@ -466,6 +516,33 @@ class LectricClient:
         """
         return is_algo_available_exact_available_algo_get.sync_detailed(client=self.client)
 
+
+    @propagate_exception
+    def exact_select(self, collection_name: str,
+                        what: str = "*",
+                        where: str = None,
+                        order_by: str = None,
+                        desc: bool = True,
+                        limit: int = 10000) -> List[Dict]:
+        """Select from collection using SQL logic
+
+        Args:
+            collection_name (str): The unique name of the collection
+            what (str, optional): what columns/fields to select. Defaults to "*".
+            where (str, optional): the where predicate. Defaults to None.
+            order_by (str, optional): order the results by this field. Defaults to None.
+            desc (bool, optional): order descending? or ascending. Defaults to True.
+            limit (int, optional): the max results to return. Defaults to 10000.
+
+        Returns:
+            List[Dict]: entries matching the query from the database
+        """
+
+        return select_entries_exact_select_post.sync_detailed(
+            client=self.client, collection_name=collection_name,
+            what=what, where=where, order_by=order_by, desc=desc, limit=limit
+        )
+
     def _pformat_collection(self, collection: Collection) -> str:
         return f"""
                 Collection name: {collection.name}
@@ -484,7 +561,7 @@ class LectricClient:
 
         Args:
             collection_name (str): The unique name of the collection
-            exact (bool, ojptional): Exact collections only? Defaults to False.
+            exact (bool, optional): Exact collections only? Defaults to False.
         """
         if exact:
             return self._exact_info(collection_name)
@@ -507,13 +584,15 @@ class LectricClient:
     @propagate_exception
     def reverse_lookup(self, collection_name: str, limit: int = None) -> List[Tuple]:
         """Get elements from the reverse lookup
+
         Args:
             collection_name (str): The unique name of the collection
-            limit (int, optional): Liit the number of outputs. Defaults to None.
+            limit (int, optional): Limit the number of outputs. Defaults to None.
 
         Returns:
-
+            List[Tuple]: The returned elements in the reverse lookup
         """
+
         return revlookup_list_exact_revlookup_list_get.sync_detailed(
                         client=self.client, name=collection_name, limit=limit)
 
@@ -567,33 +646,44 @@ class LectricClient:
         """
         return  ping_exact_ping_post.sync_detailed(client=self.client)
 
-    @propagate_exception
-    def get_schema_info(self, collection_name: str) -> List[Tuple]:
+    @propagate_exception # TODO: Hack! We should use get_collection and populate accordingly
+    def get_schema_info(self, collection_name: str) -> SchemaInfoExactSchemaInfoGetResponseSchemaInfoExactSchemaInfoGet:
         """Get the underlying storage schema
 
         Args:
             collection_name (str): The name of the collection
 
         Returns:
-            List[Tuple]: Each column in the schema with it's type
+            SchemaInfoExactSchemaInfoGetResponseSchemaInfoExactSchemaInfoGet: A representation of each column in the schema with its type
         """
         return schema_info_exact_schema_info_get.sync_detailed(client=self.client, name=collection_name)
 
     @propagate_exception
-    def get_url(self, collection_name: str, file_path: str, exact_only: bool=True,
-                access_duration_sec: float=60*60) -> str:
+    def get_url(self, path: str, access_duration_sec: float=60*60) -> str:
         """Get the URL of data stored within Lectric given a file path. Obtain the file path through a query on an collection that has a link field schema
 
         Args:
-            collection_name (str): The unique name of the collection
-            file_path (str): The file path within the collection
+            path (str): The file path within the collection
             access_duration_sec (float): How long the url will be active
         Returns:
-            str: _description_
+            str: a fully defined url
         """
 
-        if not exact_only:
-            raise NotImplementedError("Approximate local storage is a TODO")
+        return get_url_collection_get_url_get.sync_detailed(client=self.client,
+                                                    path=path, access_duration_sec=access_duration_sec)
 
-        return get_url_exact_get_url_get.sync_detailed(client=self.client, name=collection_name,
-                                                    file_path=file_path, access_duration_sec=access_duration_sec)
\ No newline at end of file
+    @propagate_exception
+    def get_collection_metadata(self, collection_name: str, is_approx: bool=False) -> CollectionMetadata:
+        """Retrieves the collection metadata, which includes the ETag stored in the form of the last updated timestamp.
+
+        Args:
+            collection_name (str): Collection Name
+            is_approx (bool): Is the collection an approximate one or not
+        Returns:
+            str: Collection Metadata
+        """
+        if is_approx:
+            raise NotImplementedError("Approximate collections not yet supported")
+        else:
+            return get_collection_metadata_exact_collection_metadata_get.sync_detailed(
+                                        client=self.client, collection_name=collection_name)
\ No newline at end of file
