@@ -1,21 +1,19 @@
-from typing import *
+from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union
 
-import attr
+from attrs import define as _attrs_define
+from attrs import field as _attrs_field
 
 from ..models.collection_object_type import CollectionObjectType
-from ..models.collection_schema import CollectionSchema
 from ..types import UNSET, Unset
 
+if TYPE_CHECKING:
+    from ..models.collection_schema import CollectionSchema
+
+
 T = TypeVar("T", bound="Collection")
 
 
-def unset_if_none(_dict: dict, key: Any) -> Union[Any, Unset]:
-    if key not in _dict or _dict[key] is None:
-        return UNSET
-    return _dict[key]
-
-
-@attr.s(auto_attribs=True)
+@_attrs_define
 class Collection:
     """
     Attributes:
@@ -27,27 +25,11 @@ class Collection:
     """
 
     name: str
-    coll_schema: CollectionSchema
+    coll_schema: "CollectionSchema"
     object_type: Union[Unset, CollectionObjectType] = CollectionObjectType.COLLECTION
     consistency_level: Union[Unset, str] = "Session"
     approx: Union[Unset, bool] = True
-    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)
-
-    def __init__(
-        self,
-        name: str,
-        coll_schema: CollectionSchema,
-        object_type: Union[Unset, CollectionObjectType] = CollectionObjectType.COLLECTION,
-        consistency_level: Union[Unset, str] = "Session",
-        approx: Union[Unset, bool] = True,
-    ):
-        """ """
-
-        self.name = name
-        self.coll_schema = coll_schema
-        self.object_type = object_type
-        self.consistency_level = consistency_level
-        self.approx = approx
+    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)
 
     def to_dict(self) -> Dict[str, Any]:
         name = self.name
@@ -79,21 +61,23 @@ class Collection:
 
     @classmethod
     def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
+        from ..models.collection_schema import CollectionSchema
+
         d = src_dict.copy()
         name = d.pop("name")
 
         coll_schema = CollectionSchema.from_dict(d.pop("coll_schema"))
 
-        _object_type = unset_if_none(d, "object_type")
+        _object_type = d.pop("object_type", UNSET)
         object_type: Union[Unset, CollectionObjectType]
         if isinstance(_object_type, Unset):
             object_type = UNSET
         else:
             object_type = CollectionObjectType(_object_type)
 
-        consistency_level = unset_if_none(d, "consistency_level")
+        consistency_level = d.pop("consistency_level", UNSET)
 
-        approx = unset_if_none(d, "approx")
+        approx = d.pop("approx", UNSET)
 
         collection = cls(
             name=name,
