@@ -1,45 +1,31 @@
-from typing import *
+from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union
 
-import attr
+from attrs import define as _attrs_define
+from attrs import field as _attrs_field
 
 from ..models.collection_schema_object_type import CollectionSchemaObjectType
-from ..models.field_schema import FieldSchema
 from ..types import UNSET, Unset
 
+if TYPE_CHECKING:
+    from ..models.field_schema import FieldSchema
+
+
 T = TypeVar("T", bound="CollectionSchema")
 
 
-def unset_if_none(_dict: dict, key: Any) -> Union[Any, Unset]:
-    if key not in _dict or _dict[key] is None:
-        return UNSET
-    return _dict[key]
-
-
-@attr.s(auto_attribs=True)
+@_attrs_define
 class CollectionSchema:
     """
     Attributes:
         object_type (Union[Unset, CollectionSchemaObjectType]):  Default: CollectionSchemaObjectType.COLLECTIONSCHEMA.
-        fields (Union[Unset, List[FieldSchema]]):
+        fields (Union[Unset, List['FieldSchema']]):
         description (Union[Unset, str]):  Default: ''.
     """
 
     object_type: Union[Unset, CollectionSchemaObjectType] = CollectionSchemaObjectType.COLLECTIONSCHEMA
-    fields: Union[Unset, List[FieldSchema]] = UNSET
+    fields: Union[Unset, List["FieldSchema"]] = UNSET
     description: Union[Unset, str] = ""
-    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)
-
-    def __init__(
-        self,
-        object_type: Union[Unset, CollectionSchemaObjectType] = CollectionSchemaObjectType.COLLECTIONSCHEMA,
-        fields: Union[Unset, List[FieldSchema]] = UNSET,
-        description: Union[Unset, str] = "",
-    ):
-        """ """
-
-        self.object_type = object_type
-        self.fields = fields
-        self.description = description
+    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)
 
     def to_dict(self) -> Dict[str, Any]:
         object_type: Union[Unset, str] = UNSET
@@ -70,8 +56,10 @@ class CollectionSchema:
 
     @classmethod
     def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
+        from ..models.field_schema import FieldSchema
+
         d = src_dict.copy()
-        _object_type = unset_if_none(d, "object_type")
+        _object_type = d.pop("object_type", UNSET)
         object_type: Union[Unset, CollectionSchemaObjectType]
         if isinstance(_object_type, Unset):
             object_type = UNSET
@@ -79,13 +67,13 @@ class CollectionSchema:
             object_type = CollectionSchemaObjectType(_object_type)
 
         fields = []
-        _fields = unset_if_none(d, "fields")
+        _fields = d.pop("fields", UNSET)
         for fields_item_data in _fields or []:
             fields_item = FieldSchema.from_dict(fields_item_data)
 
             fields.append(fields_item)
 
-        description = unset_if_none(d, "description")
+        description = d.pop("description", UNSET)
 
         collection_schema = cls(
             object_type=object_type,
