@@ -1,6 +1,7 @@
-from typing import *
+from typing import Any, Dict, List, Type, TypeVar, Union
 
-import attr
+from attrs import define as _attrs_define
+from attrs import field as _attrs_field
 
 from ..models.field_schema_object_type import FieldSchemaObjectType
 from ..types import UNSET, Unset
@@ -8,13 +9,7 @@ from ..types import UNSET, Unset
 T = TypeVar("T", bound="FieldSchema")
 
 
-def unset_if_none(_dict: dict, key: Any) -> Union[Any, Unset]:
-    if key not in _dict or _dict[key] is None:
-        return UNSET
-    return _dict[key]
-
-
-@attr.s(auto_attribs=True)
+@_attrs_define
 class FieldSchema:
     """
     Attributes:
@@ -36,29 +31,7 @@ class FieldSchema:
     description: Union[Unset, str] = ""
     dim: Union[Unset, int] = UNSET
     max_length: Union[Unset, int] = UNSET
-    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)
-
-    def __init__(
-        self,
-        name: str,
-        dtype: int,
-        object_type: Union[Unset, FieldSchemaObjectType] = FieldSchemaObjectType.FIELDSCHEMA,
-        is_primary: Union[Unset, bool] = False,
-        auto_id: Union[Unset, bool] = False,
-        description: Union[Unset, str] = "",
-        dim: Union[Unset, int] = UNSET,
-        max_length: Union[Unset, int] = UNSET,
-    ):
-        """ """
-
-        self.name = name
-        self.dtype = dtype
-        self.object_type = object_type
-        self.is_primary = is_primary
-        self.auto_id = auto_id
-        self.description = description
-        self.dim = dim
-        self.max_length = max_length
+    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)
 
     def to_dict(self) -> Dict[str, Any]:
         name = self.name
@@ -103,22 +76,22 @@ class FieldSchema:
 
         dtype = d.pop("dtype")
 
-        _object_type = unset_if_none(d, "object_type")
+        _object_type = d.pop("object_type", UNSET)
         object_type: Union[Unset, FieldSchemaObjectType]
         if isinstance(_object_type, Unset):
             object_type = UNSET
         else:
             object_type = FieldSchemaObjectType(_object_type)
 
-        is_primary = unset_if_none(d, "is_primary")
+        is_primary = d.pop("is_primary", UNSET)
 
-        auto_id = unset_if_none(d, "auto_id")
+        auto_id = d.pop("auto_id", UNSET)
 
-        description = unset_if_none(d, "description")
+        description = d.pop("description", UNSET)
 
-        dim = unset_if_none(d, "dim")
+        dim = d.pop("dim", UNSET)
 
-        max_length = unset_if_none(d, "max_length")
+        max_length = d.pop("max_length", UNSET)
 
         field_schema = cls(
             name=name,
