@@ -1,21 +1,19 @@
-from typing import *
+from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union
 
-import attr
+from attrs import define as _attrs_define
+from attrs import field as _attrs_field
 
-from ..models.query_meta_params import QueryMetaParams
 from ..models.query_params_object_type import QueryParamsObjectType
 from ..types import UNSET, Unset
 
+if TYPE_CHECKING:
+    from ..models.query_meta_params import QueryMetaParams
+
+
 T = TypeVar("T", bound="QueryParams")
 
 
-def unset_if_none(_dict: dict, key: Any) -> Union[Any, Unset]:
-    if key not in _dict or _dict[key] is None:
-        return UNSET
-    return _dict[key]
-
-
-@attr.s(auto_attribs=True)
+@_attrs_define
 class QueryParams:
     """
     Attributes:
@@ -25,21 +23,9 @@ class QueryParams:
     """
 
     metric_type: str
-    params: QueryMetaParams
+    params: "QueryMetaParams"
     object_type: Union[Unset, QueryParamsObjectType] = QueryParamsObjectType.QUERYPARAMS
-    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)
-
-    def __init__(
-        self,
-        metric_type: str,
-        params: QueryMetaParams,
-        object_type: Union[Unset, QueryParamsObjectType] = QueryParamsObjectType.QUERYPARAMS,
-    ):
-        """ """
-
-        self.metric_type = metric_type
-        self.params = params
-        self.object_type = object_type
+    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)
 
     def to_dict(self) -> Dict[str, Any]:
         metric_type = self.metric_type
@@ -64,12 +50,14 @@ class QueryParams:
 
     @classmethod
     def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
+        from ..models.query_meta_params import QueryMetaParams
+
         d = src_dict.copy()
         metric_type = d.pop("metric_type")
 
         params = QueryMetaParams.from_dict(d.pop("params"))
 
-        _object_type = unset_if_none(d, "object_type")
+        _object_type = d.pop("object_type", UNSET)
         object_type: Union[Unset, QueryParamsObjectType]
         if isinstance(_object_type, Unset):
             object_type = UNSET
