@@ -1,51 +1,38 @@
-from typing import *
+from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union
 
-import attr
+from attrs import define as _attrs_define
+from attrs import field as _attrs_field
 
-from ..models.hit import Hit
 from ..models.query_response_object_type import QueryResponseObjectType
 from ..types import UNSET, Unset
 
+if TYPE_CHECKING:
+    from ..models.hit import Hit
+
+
 T = TypeVar("T", bound="QueryResponse")
 
 
-def unset_if_none(_dict: dict, key: Any) -> Union[Any, Unset]:
-    if key not in _dict or _dict[key] is None:
-        return UNSET
-    return _dict[key]
-
-
-@attr.s(auto_attribs=True)
+@_attrs_define
 class QueryResponse:
     """
     Attributes:
-        hits (List[List[Hit]]):
+        collection_name (str):
+        hits (List[List['Hit']]):
         query_latency (float):
         total_latency (float):
         object_type (Union[Unset, QueryResponseObjectType]):  Default: QueryResponseObjectType.QUERYRESPONSE.
     """
 
-    hits: List[List[Hit]]
+    collection_name: str
+    hits: List[List["Hit"]]
     query_latency: float
     total_latency: float
     object_type: Union[Unset, QueryResponseObjectType] = QueryResponseObjectType.QUERYRESPONSE
-    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)
-
-    def __init__(
-        self,
-        hits: List[List[Hit]],
-        query_latency: float,
-        total_latency: float,
-        object_type: Union[Unset, QueryResponseObjectType] = QueryResponseObjectType.QUERYRESPONSE,
-    ):
-        """ """
-
-        self.hits = hits
-        self.query_latency = query_latency
-        self.total_latency = total_latency
-        self.object_type = object_type
+    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)
 
     def to_dict(self) -> Dict[str, Any]:
+        collection_name = self.collection_name
         hits = []
         for hits_item_data in self.hits:
             hits_item = []
@@ -66,6 +53,7 @@ class QueryResponse:
         field_dict.update(self.additional_properties)
         field_dict.update(
             {
+                "collection_name": collection_name,
                 "hits": hits,
                 "query_latency": query_latency,
                 "total_latency": total_latency,
@@ -78,7 +66,11 @@ class QueryResponse:
 
     @classmethod
     def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
+        from ..models.hit import Hit
+
         d = src_dict.copy()
+        collection_name = d.pop("collection_name")
+
         hits = []
         _hits = d.pop("hits")
         for hits_item_data in _hits:
@@ -95,7 +87,7 @@ class QueryResponse:
 
         total_latency = d.pop("total_latency")
 
-        _object_type = unset_if_none(d, "object_type")
+        _object_type = d.pop("object_type", UNSET)
         object_type: Union[Unset, QueryResponseObjectType]
         if isinstance(_object_type, Unset):
             object_type = UNSET
@@ -103,6 +95,7 @@ class QueryResponse:
             object_type = QueryResponseObjectType(_object_type)
 
         query_response = cls(
+            collection_name=collection_name,
             hits=hits,
             query_latency=query_latency,
             total_latency=total_latency,
