@@ -1,6 +1,7 @@
-from typing import *
+from typing import Any, Dict, List, Type, TypeVar, Union, cast
 
-import attr
+from attrs import define as _attrs_define
+from attrs import field as _attrs_field
 
 from ..models.query_spec_object_type import QuerySpecObjectType
 from ..types import UNSET, Unset
@@ -8,13 +9,7 @@ from ..types import UNSET, Unset
 T = TypeVar("T", bound="QuerySpec")
 
 
-def unset_if_none(_dict: dict, key: Any) -> Union[Any, Unset]:
-    if key not in _dict or _dict[key] is None:
-        return UNSET
-    return _dict[key]
-
-
-@attr.s(auto_attribs=True)
+@_attrs_define
 class QuerySpec:
     """
     Attributes:
@@ -28,21 +23,7 @@ class QuerySpec:
     collection_name: str
     object_type: Union[Unset, QuerySpecObjectType] = QuerySpecObjectType.QUERYSPEC
     output_fields: Union[Unset, List[str]] = UNSET
-    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)
-
-    def __init__(
-        self,
-        expr: str,
-        collection_name: str,
-        object_type: Union[Unset, QuerySpecObjectType] = QuerySpecObjectType.QUERYSPEC,
-        output_fields: Union[Unset, List[str]] = UNSET,
-    ):
-        """ """
-
-        self.expr = expr
-        self.collection_name = collection_name
-        self.object_type = object_type
-        self.output_fields = output_fields
+    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)
 
     def to_dict(self) -> Dict[str, Any]:
         expr = self.expr
@@ -77,14 +58,14 @@ class QuerySpec:
 
         collection_name = d.pop("collection_name")
 
-        _object_type = unset_if_none(d, "object_type")
+        _object_type = d.pop("object_type", UNSET)
         object_type: Union[Unset, QuerySpecObjectType]
         if isinstance(_object_type, Unset):
             object_type = UNSET
         else:
             object_type = QuerySpecObjectType(_object_type)
 
-        output_fields = cast(List[str], unset_if_none(d, "output_fields"))
+        output_fields = cast(List[str], d.pop("output_fields", UNSET))
 
         query_spec = cls(
             expr=expr,
